@@ -12,31 +12,49 @@ class QuerifyException(Exception):
 
 class Querify:
 
-	def __init__(self, table_name=None, custom_filters=None, allowed_columns=None, search_fields=None, search_ci=False, column_escape='`', table_escape=''):
+	def __init__(self, 
+			table,
+			columns=None,
+			filterable_columns=None, 
+			searchable_columns=None, 
+			custom_filters=None, 
+			custom_search=None, 
+			search_ci=False, 
+			column_escape='`', 
+			table_escape=''
+		):
 
-		self.table_name = table_name if (
-			table_name and isinstance(table_name,str)
-		) else None
-		
-		self.allowed_columns = allowed_columns if (
-			allowed_columns and isinstance(allowed_columns,list)
+		self.table_name = table if (
+			table and isinstance(table,str)
 		) else None
 
-		self.custom_filters =custom_filters if (
+		if not self.table_name:
+			raise QuerifyException("Table name is invalid")
+
+		self.columns = columns if (
+			columns and isinstance(columns,list)
+		) else None
+
+		self.filterable_columns = filterable_columns if (
+			filterable_columns and isinstance(filterable_columns,list)
+		) else None
+
+		self.searchable_columns = searchable_columns if (
+			searchable_columns and isinstance(searchable_columns,list)
+		) else None
+
+		self.custom_filters = custom_filters if (
 			custom_filters and isinstance(custom_filters,dict)
 		) else None
 
-		if isinstance(search_fields, tuple):
-			search_fields = [*search_fields]
-		
-		self.search_fields = search_fields if (
-			search_fields and isinstance(search_fields,list)
+		self.custom_search = custom_search if (
+			custom_search and (isinstance(custom_search, dict) or callable(custom_search))
 		) else None
-
+		
 		self.search_ci = search_ci
 		self.column_escape = column_escape
 		self.table_escape = table_escape
-
+		
 		pass
 
 	
@@ -61,7 +79,7 @@ class Querify:
 				and f in custom_filters[col][filter_type]
 			):
 				filters.append(
-					custom_filters[col][filter_type][f]
+					"(%s)" % custom_filters[col][filter_type][f]
 				)
 				continue
 			
@@ -77,7 +95,9 @@ class Querify:
 
 			if filters and isinstance(filters, dict):
 				for col in filters:
-					if self.allowed_columns and col not in self.allowed_columns:
+					if not filters[col]:
+						continue
+					if self.filterable_columns and col not in self.filterable_columns:
 						raise QuerifyException("Column not allowed: %s" % col)
 
 					if "positive" in filters[col] and isinstance(filters[col]["positive"], list):
@@ -90,23 +110,29 @@ class Querify:
 							self.parse_filter(filters, col, "negative")
 						)
 			
-			if search and self.search_fields:
-				search_fields = self.search_fields
-				search_conditions = []
-				for sf in search_fields:
-					search_ci = ' COLLATE UTF8_GENERAL_CI ' if self.search_ci else '' # Case Insesitive Search
-					search_conditions.append(
-						"{}{}{}{} LIKE '%{}%'".format(self.column_escape, self.escape_string(sf), self.column_escape, search_ci, self.escape_string(search))
+			if search and (self.searchable_columns or callable(self.custom_search)):
+				if callable(self.custom_search):
+					conditions.append(
+						"({})".format(self.custom_search(search))
+					)
+				else:
+					search_fields = self.searchable_columns
+					search_conditions = []
+
+					for sf in search_fields:
+						search_ci = ' COLLATE UTF8_GENERAL_CI ' if self.search_ci else '' # Case Insesitive Search
+						search_conditions.append(
+							"{}{}{}{} LIKE '%%{}%%'".format(self.column_escape, self.escape_string(sf), self.column_escape, search_ci, self.escape_string(search))
+						)
+
+					conditions.append(
+						"({})".format(' OR '.join(search_conditions))
 					)
 
-				conditions.append(
-					"({})".format(' OR '.join(search_conditions))
-				)
-
-			if sort and isinstance(sort, dict) and "type" in sort and "column" in sort:
+			if sort and isinstance(sort, dict) and "type" in sort and "column" in sort and sort["column"]:
 				sort_type = "DESC" if not sort["type"] else "ASC"
 				sort_column = sort["column"]
-				if self.allowed_columns and sort_column not in self.allowed_columns:
+				if self.filterable_columns and sort_column not in self.filterable_columns:
 					raise QuerifyException("Column not allowed: %s" % sort_column)
 
 				others.append(
@@ -125,7 +151,14 @@ class Querify:
 
 			if self.table_name:
 				hasConditions =  (" WHERE " if len(conditions) else "")
-				sql = "SELECT * FROM {}{}{}{}{}".format(self.table_escape, self.table_name, self.table_escape, hasConditions, sql)
+				sql = "SELECT {} FROM {}{}{}{}{}".format(
+					', '.join([
+						'{}{}{}'.format(self.column_escape, column, self.column_escape) for column in self.columns
+					]) if self.columns else '*',
+					self.table_escape, self.table_name, self.table_escape,
+					hasConditions, 
+					sql
+				)
 				total_sql = "SELECT count(*) as total FROM {}{}{}{}{}".format(self.table_escape, self.table_name, self.table_escape, hasConditions, conditions_sql)
 
 			return [sql, total_sql]
@@ -133,9 +166,10 @@ class Querify:
 	
 ''' Test
 querify = Querify(
-	table_name="users",
-	search_fields=('first','last','country'), # Searchable fields if we support searching 
-	allowed_columns=['first','last','age','country','phone'], # Allowed filterable columns to prevent malicious injections
+	table="users",
+	columns=['id','username','email','first_name','last_name',]
+	searchable_columns=('first','last','country'), # Searchable fields if we support searching 
+	filterable_columns=['first','last','age','country','phone'], # Allowed filterable columns to prevent malicious injections
 	custom_filters={  # Parsers for custom filters | By default a positive filter would be something like this "`age` = '[filter]'"
 		'age':{
 			'positive': {
